@@ -52,6 +52,17 @@ std::string escapeString( const char *s )
 					unsigned short val = (s[2] & 0x3F) + ((s[1] & 0x3F) << 6) + ((s[0] & 0x0F) << 12);
 					oss << "\\u" << std::setfill('0') << std::setw(4) << std::hex << val << std::dec;
 					s += 2;
+				} else if( (ch & 0xF8) == 0xF0 && s[1] && s[2] && s[3] ) {
+					// 11110xxx 10xxxxxx 10xxxxxx 10xxxxxx
+					unsigned long val = (s[3] & 0x3F) + ((s[2] & 0x3F) << 6) + ((s[1] & 0x3F) << 12) + ((s[0] & 0x07) << 18);
+					// Per JSON spec, encode as UTF-16 https://en.wikipedia.org/wiki/UTF-16#Code_points_from_U+010000_to_U+10FFFF
+					// Could probably do directly from UTF-8 for improved performance
+					unsigned long U = val - 0x10000;
+					unsigned short hi = (U >> 10) + 0xD800;
+					unsigned short lo = (U & 0x3FF) + 0xDC00;
+					oss << "\\u" << std::setfill('0') << std::setw(4) << std::hex << hi << std::dec;
+					oss << "\\u" << std::setfill('0') << std::setw(4) << std::hex << lo << std::dec;
+					s += 3;
 				} else {
 					error("unable to escape string. String is not utf8\n");
 				}
@@ -74,7 +85,8 @@ std::string toJSON2( SEXP x, int indent, int indent_amount )
 
 	int i = 0;
 	int n = length(x);
-	SEXP names = GET_NAMES(x);
+	SEXP names;
+	PROTECT( names = GET_NAMES(x) );
 
 	//int container = NO_CONTAINER;
 	std::string container_closer;
@@ -202,7 +214,7 @@ std::string toJSON2( SEXP x, int indent, int indent_amount )
 		default:
 			error("unable to convert R type %i to JSON\n", TYPEOF(x));
 	}
-	UNPROTECT(1);
+	UNPROTECT(2);
 	if( !container_closer.empty() ) {
 		indent -= indent_amount;
 		if( indent_amount > 0 ) { oss << "\n"; }
